//------------------------------------------------------------------------------
//
//  npgl.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npgl.h"

#include <stdlib.h>

#include "../npctrl.h"

#include "npglut.h"


static float angle = 0.0, ratio;
static float x = 0.0f, y = 1.75f, z=5.0f;
static float lx = 0.0f, ly = 0.0f, lz = -1.0f;
static GLint snowman_display_list;


//------------------------------------------------------------------------------
void npGLDrawScene (void* dataRef)
{
	int i = 0;
	int j = 0;

	pData data = (pData) dataRef;

	NPplotPtr camPlot = data->map.plot[1];
	NPcameraPtr camData = camPlot->data;

	npCtrlUpdateData (dataRef);
	npCtrlUpdateEngine (dataRef);

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

// Draw ground

	glColor3f(0.9f, 0.9f, 0.9f);
	glBegin(GL_QUADS);
		glVertex3f(-100.0f, 0.0f, -100.0f);
		glVertex3f(-100.0f, 0.0f,  100.0f);
		glVertex3f( 100.0f, 0.0f,  100.0f);
		glVertex3f( 100.0f, 0.0f, -100.0f);
	glEnd();

// Draw 36 SnowMen

	for(i = -3; i < 3; i++)
		for(j=-3; j < 3; j++) {
			glPushMatrix();
			glTranslatef(i*10.0f,0,j * 10.0f);
			glCallList(snowman_display_list);
			glPopMatrix();
		}
	
	glLoadIdentity();
	gluLookAt (	camPlot->translate.x, 
				camPlot->translate.y,
				camPlot->translate.z, 
				camPlot->translate.x + camPlot->rotate.x,
				camPlot->translate.y + camPlot->rotate.y,
				camPlot->translate.z + camPlot->rotate.z,
				0.0f, 1.0f, 0.0f );
			//	camPlot->rotate.x,
			//	camPlot->rotate.y,
			//	camPlot->rotate.z );
}


//------------------------------------------------------------------------------
void npGLResizeScene (int width, int height)
{
	pData data = npDataGetDataRef();

	NPplotPtr camPlot = data->map.plot[1];
	NPcameraPtr camData = camPlot->data;

	if (height == 0)			// Prevent a divide by zero
		height = 1;

	ratio = 1.0f * width / height;

	// Reset the coordinate system before modifying
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, width, height);

	// Set the clipping volume
	gluPerspective (45.0f, ratio, 1.0f, 1000.0f);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt (	camPlot->translate.x, 
				camPlot->translate.y,
				camPlot->translate.z, 
				camPlot->translate.x + camPlot->rotate.x,
				camPlot->translate.y + camPlot->rotate.y,
				camPlot->translate.z + camPlot->rotate.z,
				0.0f, 1.0f, 0.0f );
			//	camPlot->rotate.x,
			//	camPlot->rotate.y,
			//	camPlot->rotate.z );
}


//------------------------------------------------------------------------------
void drawSnowMan() 
{
	glColor3f(0.0f, 1.0f, 1.0f);

// Draw Body	
	glTranslatef(0.0f ,0.75f, 0.0f);
	npGLSolidSphere(0.75f,20,20);

// Draw Head
	glTranslatef(0.0f, 1.0f, 0.0f);
	npGLSolidSphere(0.25f,20,20);

// Draw Eyes
	glPushMatrix();
	glColor3f(0.0f,0.0f,0.0f);
	glTranslatef(0.05f, 0.10f, 0.18f);
	npGLSolidSphere(0.05f,10,10);
	glTranslatef(-0.1f, 0.0f, 0.0f);
	npGLSolidSphere(0.05f,10,10);
	glPopMatrix();

// Draw Nose
	glColor3f(1.0f, 0.5f , 0.5f);
	glRotatef(0.0f,1.0f, 0.0f, 0.0f);
	npGLSolidCone(0.08f,0.5f,10,2);
}


//------------------------------------------------------------------------------
GLuint createDL()
{
	GLuint snowManDL;

	// Create the id for the list
	snowManDL = glGenLists(1);

	// start list
	glNewList(snowManDL,GL_COMPILE);

	// call the function that contains the rendering commands
	drawSnowMan();

	// endList
	glEndList();

	return(snowManDL);
}


//------------------------------------------------------------------------------
void npInitGL() 
{
//	glShadeModel (GL_SMOOTH);
//	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
//	glClearDepth (1.0f);

	glEnable (GL_DEPTH_TEST);

//	glDepthFunc (GL_LEQUAL);
//	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

	snowman_display_list = createDL();
}


//------------------------------------------------------------------------------
void orientMe (float ang)
{
	lx = (float)sin(ang);
	lz = (float)-cos(ang);
	glLoadIdentity();
	gluLookAt(x, y, z, 
		      x + lx,y + ly,z + lz,
			  0.0f,1.0f,0.0f);
}


//------------------------------------------------------------------------------
void moveMeFlat(int i) {
	x = x + i*(lx)*0.1f;
	z = z + i*(lz)*0.1f;
	glLoadIdentity();
	gluLookAt(x, y, z,
		      x + lx,y + ly,z + lz,
			  0.0f,1.0f,0.0f);
}


//------------------------------------------------------------------------------
void npDrawGrid (NPgridPtr grid)
{

}

